@@ -19,6 +19,7 @@ module Network.HTTP.Client.Manager
     , proxyEnvironment
     , proxyEnvironmentNamed
     , defaultProxy
+    , dropProxyAuthSecure
     ) where
 
 #if !MIN_VERSION_base(4,6,0)
@@ -54,6 +55,7 @@ import Network.HTTP.Types (status200)
 import Network.HTTP.Client.Types
 import Network.HTTP.Client.Connection
 import Network.HTTP.Client.Headers (parseStatusHeaders)
+import Network.HTTP.Client.Request (username, password, applyBasicProxyAuth)
 import Control.Concurrent.MVar (MVar, takeMVar, tryPutMVar, newEmptyMVar)
 import System.Environment (getEnvironment)
 import qualified Network.URI as U
@@ -384,6 +386,18 @@ getConnDest req =
         Just p -> (True, S8.unpack (proxyHost p), proxyPort p)
         Nothing -> (False, S8.unpack $ host req, port req)
 
+-- | Drop the Proxy-Authorization header from the request if we're using a
+-- secure proxy.
+dropProxyAuthSecure :: Request -> Request
+dropProxyAuthSecure req
+    | secure req && useProxy = req
+        { requestHeaders = filter (\(k, _) -> k /= "Proxy-Authorization")
+                                  (requestHeaders req)
+        }
+    | otherwise = req
+  where
+    (useProxy, _, _) = getConnDest req
+
 getConn :: Request
         -> Manager
         -> IO (ConnRelease, Connection, ManagedConn)
@@ -412,12 +426,15 @@ getConn req m
             (True, True) ->
                 let ultHost = host req
                     ultPort = port req
+                    proxyAuthorizationHeader = maybe "" (\h -> S8.concat ["Proxy-Authorization: ", h, "\r\n"]) . lookup "Proxy-Authorization" $ requestHeaders req
                     connstr = S8.concat
                         [ "CONNECT "
                         , ultHost
                         , ":"
                         , S8.pack $ show ultPort
-                        , " HTTP/1.1\r\n\r\n"
+                        , " HTTP/1.1\r\n"
+                        , proxyAuthorizationHeader
+                        , "\r\n"
                         ]
                     parse conn = do
                         sh@(StatusHeaders status _ _) <- parseStatusHeaders conn Nothing Nothing
@@ -486,14 +503,11 @@ envHelper :: Text -> EnvHelper -> IO (Request -> Request)
 envHelper name eh = do
     env <- getEnvironment
     case lookup (T.unpack name) env of
-        Nothing -> return $
-            case eh of
-                EHFromRequest -> id
-                EHNoProxy     -> \req -> req { proxy = Nothing }
-                EHUseProxy p  -> \req -> req { proxy = Just p  }
+        Nothing  -> return noEnvProxy
+        Just ""  -> return noEnvProxy
         Just str -> do
             let invalid = throwIO $ InvalidProxyEnvironmentVariable name (T.pack str)
-            p <- maybe invalid return $ do
+            (p, muserpass) <- maybe invalid return $ do
                 uri <- case U.parseURI str of
                     Just u | U.uriScheme u == "http:" -> return u
                     _ -> U.parseURI $ "http://" ++ str
@@ -504,7 +518,13 @@ envHelper name eh = do
                 guard $ null $ U.uriFragment uri
 
                 auth <- U.uriAuthority uri
-                guard $ null $ U.uriUserInfo auth
+                let muserpass =
+                        if null authInfo
+                            then Nothing
+                            else Just ( S8.pack $ username authInfo
+                                      , S8.pack $ password authInfo
+                                      )
+                    authInfo = U.uriUserInfo auth
 
                 port <-
                     case U.uriPort auth of
@@ -515,5 +535,11 @@ envHelper name eh = do
                                 _ -> Nothing
                         _ -> Nothing
 
-                Just $ Proxy (S8.pack $ U.uriRegName auth) port
-            return $ \req -> req { proxy = Just p }
+                Just $ (Proxy (S8.pack $ U.uriRegName auth) port, muserpass)
+            return $ \req ->
+                maybe id (uncurry applyBasicProxyAuth) muserpass
+                req { proxy = Just p }
+    where noEnvProxy = case eh of
+            EHFromRequest -> id
+            EHNoProxy     -> \req -> req { proxy = Nothing }
+            EHUseProxy p  -> \req -> req { proxy = Just p  }
